# -*- coding: utf-8 -*-
"""
Created on Wed Mar 14 19:58:29 2018

@author: sarth
"""

from setuptools import setup

setup(
name = 'SAFMI_AFM',
version = '1.0.1',
packages=['SAFMI/UserImageProcessing', 'SAFMI/ImageDisorderPrediction']
)
